@@ -22,6 +22,7 @@ from nova.compute import api as compute
 from nova.policies import lock_server as ls_policies
 
 
+@validation.validated
 class LockServerController(wsgi.Controller):
     def __init__(self):
         super(LockServerController, self).__init__()
@@ -41,8 +42,10 @@ class LockServerController(wsgi.Controller):
                     target={'user_id': instance.user_id,
                             'project_id': instance.project_id})
         reason = None
-        if (api_version_request.is_supported(req, min_version='2.73') and
-            body['lock'] is not None):
+        if (
+            api_version_request.is_supported(req, '2.73') and
+            body['lock'] is not None
+        ):
             reason = body['lock'].get('locked_reason')
         self.compute_api.lock(context, instance, reason=reason)
 
