@@ -27,7 +27,7 @@ from webob import exc
 from nova.api.openstack import api_version_request
 from nova.api.openstack import common
 from nova.api.openstack.compute import helpers
-from nova.api.openstack.compute.schemas import servers as schema_servers
+from nova.api.openstack.compute.schemas import servers as schema
 from nova.api.openstack.compute.views import servers as views_servers
 from nova.api.openstack import wsgi
 from nova.api import validation
@@ -45,7 +45,6 @@ from nova.policies import servers as server_policies
 from nova import utils
 
 TAG_SEARCH_FILTERS = ('tags', 'tags-any', 'not-tags', 'not-tags-any')
-PARTIAL_CONSTRUCT_FOR_CELL_DOWN_MIN_VERSION = '2.69'
 PAGING_SORTING_PARAMS = ('sort_key', 'sort_dir', 'limit', 'marker')
 
 CONF = nova.conf.CONF
@@ -114,11 +113,11 @@ class ServersController(wsgi.Controller):
         self.compute_api = compute.API()
 
     @wsgi.expected_errors((400, 403))
-    @validation.query_schema(schema_servers.query_params_v275, '2.75')
-    @validation.query_schema(schema_servers.query_params_v273, '2.73', '2.74')
-    @validation.query_schema(schema_servers.query_params_v266, '2.66', '2.72')
-    @validation.query_schema(schema_servers.query_params_v226, '2.26', '2.65')
-    @validation.query_schema(schema_servers.query_params_v21, '2.1', '2.25')
+    @validation.query_schema(schema.query_params_v275, '2.75')
+    @validation.query_schema(schema.query_params_v273, '2.73', '2.74')
+    @validation.query_schema(schema.query_params_v266, '2.66', '2.72')
+    @validation.query_schema(schema.query_params_v226, '2.26', '2.65')
+    @validation.query_schema(schema.query_params_v21, '2.1', '2.25')
     def index(self, req):
         """Returns a list of server names and ids for a given user."""
         context = req.environ['nova.context']
@@ -130,11 +129,11 @@ class ServersController(wsgi.Controller):
         return servers
 
     @wsgi.expected_errors((400, 403))
-    @validation.query_schema(schema_servers.query_params_v275, '2.75')
-    @validation.query_schema(schema_servers.query_params_v273, '2.73', '2.74')
-    @validation.query_schema(schema_servers.query_params_v266, '2.66', '2.72')
-    @validation.query_schema(schema_servers.query_params_v226, '2.26', '2.65')
-    @validation.query_schema(schema_servers.query_params_v21, '2.1', '2.25')
+    @validation.query_schema(schema.query_params_v275, '2.75')
+    @validation.query_schema(schema.query_params_v273, '2.73', '2.74')
+    @validation.query_schema(schema.query_params_v266, '2.66', '2.72')
+    @validation.query_schema(schema.query_params_v226, '2.26', '2.65')
+    @validation.query_schema(schema.query_params_v21, '2.1', '2.25')
     def detail(self, req):
         """Returns a list of server details for a given user."""
         context = req.environ['nova.context']
@@ -147,8 +146,7 @@ class ServersController(wsgi.Controller):
 
     @staticmethod
     def _is_cell_down_supported(req, search_opts):
-        cell_down_support = api_version_request.is_supported(
-            req, min_version=PARTIAL_CONSTRUCT_FOR_CELL_DOWN_MIN_VERSION)
+        cell_down_support = api_version_request.is_supported(req, '2.69')
 
         if cell_down_support:
             # NOTE(tssurya): Minimal constructs would be returned from the down
@@ -185,7 +183,7 @@ class ServersController(wsgi.Controller):
 
         for search_opt in search_opts:
             if (search_opt in
-                schema_servers.JOINED_TABLE_QUERY_PARAMS_SERVERS.keys() or
+                schema.JOINED_TABLE_QUERY_PARAMS_SERVERS.keys() or
                     search_opt.startswith('_')):
                 msg = _("Invalid filter field: %s.") % search_opt
                 raise exc.HTTPBadRequest(explanation=msg)
@@ -201,7 +199,7 @@ class ServersController(wsgi.Controller):
             states = common.task_and_vm_state_from_status(statuses)
             vm_state, task_state = states
             if not vm_state and not task_state:
-                if api_version_request.is_supported(req, min_version='2.38'):
+                if api_version_request.is_supported(req, '2.38'):
                     msg = _('Invalid status value')
                     raise exc.HTTPBadRequest(explanation=msg)
 
@@ -265,7 +263,7 @@ class ServersController(wsgi.Controller):
                 msg = _("Only administrators may list deleted instances")
                 raise exc.HTTPForbidden(explanation=msg)
 
-        if api_version_request.is_supported(req, min_version='2.26'):
+        if api_version_request.is_supported(req, '2.26'):
             for tag_filter in TAG_SEARCH_FILTERS:
                 if tag_filter in search_opts:
                     search_opts[tag_filter] = search_opts[
@@ -301,9 +299,9 @@ class ServersController(wsgi.Controller):
 
         limit, marker = common.get_limit_and_marker(req)
         sort_keys, sort_dirs = common.get_sort_params(req.params)
-        blacklist = schema_servers.SERVER_LIST_IGNORE_SORT_KEY
-        if api_version_request.is_supported(req, min_version='2.73'):
-            blacklist = schema_servers.SERVER_LIST_IGNORE_SORT_KEY_V273
+        blacklist = schema.SERVER_LIST_IGNORE_SORT_KEY
+        if api_version_request.is_supported(req, '2.73'):
+            blacklist = schema.SERVER_LIST_IGNORE_SORT_KEY_V273
         sort_keys, sort_dirs = remove_invalid_sort_keys(
             context, sort_keys, sort_dirs, blacklist, ('host', 'node'))
 
@@ -459,16 +457,16 @@ class ServersController(wsgi.Controller):
         return objects.NetworkRequestList(objects=networks)
 
     @wsgi.expected_errors(404)
+    @validation.query_schema(schema.show_query)
     def show(self, req, id):
         """Returns server details by server id."""
         context = req.environ['nova.context']
-        cell_down_support = api_version_request.is_supported(
-            req, min_version=PARTIAL_CONSTRUCT_FOR_CELL_DOWN_MIN_VERSION)
-        show_server_groups = api_version_request.is_supported(
-            req, min_version='2.71')
+        cell_down_support = api_version_request.is_supported(req, '2.69')
+        show_server_groups = api_version_request.is_supported(req, '2.71')
 
         instance = self._get_server(
             context, req, id, is_detail=True,
+            columns_to_join=['services'],
             cell_down_support=cell_down_support)
         context.can(server_policies.SERVERS % 'show',
                     target={'project_id': instance.project_id})
@@ -486,7 +484,7 @@ class ServersController(wsgi.Controller):
         :param target: The target dict for ``context.can`` policy checks
         :param server_dict: The POST /servers request body "server" entry
         :param create_kwargs: dict that gets populated by this method and
-            passed to nova.comptue.api.API.create()
+            passed to nova.compute.api.API.create()
         :raises: webob.exc.HTTPBadRequest if the request parameters are invalid
         :raises: nova.exception.Forbidden if a policy check fails
         """
@@ -537,7 +535,7 @@ class ServersController(wsgi.Controller):
         :param target: The target dict for ``context.can`` policy checks
         :param server_dict: The POST /servers request body "server" entry
         :param create_kwargs: dict that gets populated by this method and
-            passed to nova.comptue.api.API.create()
+            passed to nova.compute.api.API.create()
         :raises: webob.exc.HTTPBadRequest if the request parameters are invalid
         :raises: nova.exception.Forbidden if a policy check fails
         """
@@ -643,7 +641,7 @@ class ServersController(wsgi.Controller):
         :param target: The target dict for ``context.can`` policy checks
         :param server_dict: The POST /servers request body "server" entry
         :param create_kwargs: dict that gets populated by this method and
-            passed to nova.comptue.api.API.create()
+            passed to nova.compute.api.API.create()
         :param host: Forced host of availability_zone
         :param node: Forced node of availability_zone
         :raise: webob.exc.HTTPBadRequest if the request parameters are invalid
@@ -665,19 +663,20 @@ class ServersController(wsgi.Controller):
 
     @wsgi.response(202)
     @wsgi.expected_errors((400, 403, 409))
-    @validation.schema(schema_servers.create_v20, '2.0', '2.0')
-    @validation.schema(schema_servers.create, '2.1', '2.18')
-    @validation.schema(schema_servers.create_v219, '2.19', '2.31')
-    @validation.schema(schema_servers.create_v232, '2.32', '2.32')
-    @validation.schema(schema_servers.create_v233, '2.33', '2.36')
-    @validation.schema(schema_servers.create_v237, '2.37', '2.41')
-    @validation.schema(schema_servers.create_v242, '2.42', '2.51')
-    @validation.schema(schema_servers.create_v252, '2.52', '2.56')
-    @validation.schema(schema_servers.create_v257, '2.57', '2.62')
-    @validation.schema(schema_servers.create_v263, '2.63', '2.66')
-    @validation.schema(schema_servers.create_v267, '2.67', '2.73')
-    @validation.schema(schema_servers.create_v274, '2.74', '2.89')
-    @validation.schema(schema_servers.create_v290, '2.90')
+    @validation.schema(schema.create_v20, '2.0', '2.0')
+    @validation.schema(schema.create, '2.1', '2.18')
+    @validation.schema(schema.create_v219, '2.19', '2.31')
+    @validation.schema(schema.create_v232, '2.32', '2.32')
+    @validation.schema(schema.create_v233, '2.33', '2.36')
+    @validation.schema(schema.create_v237, '2.37', '2.41')
+    @validation.schema(schema.create_v242, '2.42', '2.51')
+    @validation.schema(schema.create_v252, '2.52', '2.56')
+    @validation.schema(schema.create_v257, '2.57', '2.62')
+    @validation.schema(schema.create_v263, '2.63', '2.66')
+    @validation.schema(schema.create_v267, '2.67', '2.73')
+    @validation.schema(schema.create_v274, '2.74', '2.89')
+    @validation.schema(schema.create_v290, '2.90', '2.93')
+    @validation.schema(schema.create_v294, '2.94')
     def create(self, req, body):
         """Creates a new server for a given user."""
         context = req.environ['nova.context']
@@ -685,10 +684,10 @@ class ServersController(wsgi.Controller):
         password = self._get_server_admin_password(server_dict)
         name = common.normalize_name(server_dict['name'])
         description = name
-        if api_version_request.is_supported(req, min_version='2.19'):
+        if api_version_request.is_supported(req, '2.19'):
             description = server_dict.get('description')
         hostname = None
-        if api_version_request.is_supported(req, min_version='2.90'):
+        if api_version_request.is_supported(req, '2.90'):
             hostname = server_dict.get('hostname')
 
         # Arguments to be passed to instance create function
@@ -729,7 +728,7 @@ class ServersController(wsgi.Controller):
 
         availability_zone = server_dict.pop("availability_zone", None)
 
-        if api_version_request.is_supported(req, min_version='2.52'):
+        if api_version_request.is_supported(req, '2.52'):
             create_kwargs['tags'] = server_dict.get('tags')
 
         helpers.translate_attributes(helpers.CREATE,
@@ -761,7 +760,7 @@ class ServersController(wsgi.Controller):
             availability_zone = self._validate_host_availability_zone(
                 context, availability_zone, host)
 
-        if api_version_request.is_supported(req, min_version='2.74'):
+        if api_version_request.is_supported(req, '2.74'):
             self._process_hosts_for_create(context, target, server_dict,
                                            create_kwargs, host, node)
 
@@ -903,10 +902,11 @@ class ServersController(wsgi.Controller):
             self.compute_api.delete(context, instance)
 
     @wsgi.expected_errors(404)
-    @validation.schema(schema_servers.update_v20, '2.0', '2.0')
-    @validation.schema(schema_servers.update, '2.1', '2.18')
-    @validation.schema(schema_servers.update_v219, '2.19', '2.89')
-    @validation.schema(schema_servers.update_v290, '2.90')
+    @validation.schema(schema.update_v20, '2.0', '2.0')
+    @validation.schema(schema.update, '2.1', '2.18')
+    @validation.schema(schema.update_v219, '2.19', '2.89')
+    @validation.schema(schema.update_v290, '2.90', '2.93')
+    @validation.schema(schema.update_v294, '2.94')
     def update(self, req, id, body):
         """Update server then pass on to version-specific controller."""
 
@@ -916,8 +916,7 @@ class ServersController(wsgi.Controller):
         ctxt.can(server_policies.SERVERS % 'update',
                  target={'user_id': instance.user_id,
                          'project_id': instance.project_id})
-        show_server_groups = api_version_request.is_supported(
-             req, min_version='2.71')
+        show_server_groups = api_version_request.is_supported(req, '2.71')
 
         server = body['server']
 
@@ -940,8 +939,7 @@ class ServersController(wsgi.Controller):
 
             # NOTE(gmann): Starting from microversion 2.75, PUT and Rebuild
             # API response will show all attributes like GET /servers API.
-            show_all_attributes = api_version_request.is_supported(
-                req, min_version='2.75')
+            show_all_attributes = api_version_request.is_supported(req, '2.75')
             extend_address = show_all_attributes
             show_AZ = show_all_attributes
             show_config_drive = show_all_attributes
@@ -979,6 +977,8 @@ class ServersController(wsgi.Controller):
     @wsgi.response(204)
     @wsgi.expected_errors((400, 404, 409))
     @wsgi.action('confirmResize')
+    @validation.schema(schema.confirm_resize)
+    @validation.response_body_schema(schema.confirm_resize_response)
     def _action_confirm_resize(self, req, id, body):
         context = req.environ['nova.context']
         instance = self._get_server(context, req, id)
@@ -1001,6 +1001,8 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((400, 404, 409))
     @wsgi.action('revertResize')
+    @validation.schema(schema.revert_resize)
+    @validation.response_body_schema(schema.revert_resize_response)
     def _action_revert_resize(self, req, id, body):
         context = req.environ['nova.context']
         instance = self._get_server(context, req, id)
@@ -1023,7 +1025,8 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((404, 409))
     @wsgi.action('reboot')
-    @validation.schema(schema_servers.reboot)
+    @validation.schema(schema.reboot)
+    @validation.response_body_schema(schema.reboot_response)
     def _action_reboot(self, req, id, body):
 
         reboot_type = body['reboot']['type'].upper()
@@ -1044,7 +1047,11 @@ class ServersController(wsgi.Controller):
         """Begin the resize process with given instance/flavor."""
         context = req.environ["nova.context"]
         instance = self._get_server(context, req, instance_id,
-                                    columns_to_join=['services'])
+                                    columns_to_join=['services', 'resources',
+                                                     'pci_requests',
+                                                     'pci_devices',
+                                                     'trusted_certs',
+                                                     'vcpu_model'])
         context.can(server_policies.SERVERS % 'resize',
                     target={'user_id': instance.user_id,
                             'project_id': instance.project_id})
@@ -1086,6 +1093,10 @@ class ServersController(wsgi.Controller):
         except exception.Invalid:
             msg = _("Invalid instance image.")
             raise exc.HTTPBadRequest(explanation=msg)
+        except (
+            exception.ForbiddenSharesNotSupported,
+            exception.ForbiddenWithShare) as e:
+            raise exc.HTTPConflict(explanation=e.format_message())
 
     @wsgi.response(204)
     @wsgi.expected_errors((404, 409))
@@ -1127,7 +1138,8 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((400, 401, 403, 404, 409))
     @wsgi.action('resize')
-    @validation.schema(schema_servers.resize)
+    @validation.schema(schema.resize)
+    @validation.response_body_schema(schema.resize_response)
     def _action_resize(self, req, id, body):
         """Resizes a given instance to the flavor size requested."""
         resize_dict = body['resize']
@@ -1141,13 +1153,41 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((400, 403, 404, 409))
     @wsgi.action('rebuild')
-    @validation.schema(schema_servers.rebuild_v20, '2.0', '2.0')
-    @validation.schema(schema_servers.rebuild, '2.1', '2.18')
-    @validation.schema(schema_servers.rebuild_v219, '2.19', '2.53')
-    @validation.schema(schema_servers.rebuild_v254, '2.54', '2.56')
-    @validation.schema(schema_servers.rebuild_v257, '2.57', '2.62')
-    @validation.schema(schema_servers.rebuild_v263, '2.63', '2.89')
-    @validation.schema(schema_servers.rebuild_v290, '2.90')
+    @validation.schema(schema.rebuild_v20, '2.0', '2.0')
+    @validation.schema(schema.rebuild, '2.1', '2.18')
+    @validation.schema(schema.rebuild_v219, '2.19', '2.53')
+    @validation.schema(schema.rebuild_v254, '2.54', '2.56')
+    @validation.schema(schema.rebuild_v257, '2.57', '2.62')
+    @validation.schema(schema.rebuild_v263, '2.63', '2.89')
+    @validation.schema(schema.rebuild_v290, '2.90', '2.93')
+    @validation.schema(schema.rebuild_v294, '2.94')
+    @validation.response_body_schema(schema.rebuild_response, '2.0', '2.8')
+    @validation.response_body_schema(
+        schema.rebuild_response_v29, '2.9', '2.18')
+    @validation.response_body_schema(
+        schema.rebuild_response_v219, '2.19', '2.25')
+    @validation.response_body_schema(
+        schema.rebuild_response_v226, '2.26', '2.45')
+    @validation.response_body_schema(
+        schema.rebuild_response_v246, '2.46', '2.53')
+    @validation.response_body_schema(
+        schema.rebuild_response_v254, '2.54', '2.56')
+    @validation.response_body_schema(
+        schema.rebuild_response_v257, '2.57', '2.62')
+    @validation.response_body_schema(
+        schema.rebuild_response_v263, '2.63', '2.70')
+    @validation.response_body_schema(
+        schema.rebuild_response_v271, '2.71', '2.72')
+    @validation.response_body_schema(
+        schema.rebuild_response_v273, '2.73', '2.74')
+    @validation.response_body_schema(
+        schema.rebuild_response_v275, '2.75', '2.95')
+    @validation.response_body_schema(
+        schema.rebuild_response_v296, '2.96', '2.97')
+    @validation.response_body_schema(
+        schema.rebuild_response_v298, '2.98', '2.99')
+    @validation.response_body_schema(
+        schema.rebuild_response_v2100, '2.100')
     def _action_rebuild(self, req, id, body):
         """Rebuild an instance with the given attributes."""
         rebuild_dict = body['rebuild']
@@ -1157,7 +1197,12 @@ class ServersController(wsgi.Controller):
         password = self._get_server_admin_password(rebuild_dict)
 
         context = req.environ['nova.context']
-        instance = self._get_server(context, req, id)
+        instance = self._get_server(context, req, id,
+                                    columns_to_join=['trusted_certs',
+                                                     'pci_requests',
+                                                     'pci_devices',
+                                                     'resources',
+                                                     'migration_context'])
         target = {'user_id': instance.user_id,
                   'project_id': instance.project_id}
         context.can(server_policies.SERVERS % 'rebuild', target=target)
@@ -1172,22 +1217,21 @@ class ServersController(wsgi.Controller):
         helpers.translate_attributes(helpers.REBUILD, rebuild_dict, kwargs)
 
         if (
-            api_version_request.is_supported(req, min_version='2.54') and
+            api_version_request.is_supported(req, '2.54') and
             'key_name' in rebuild_dict
         ):
             kwargs['key_name'] = rebuild_dict.get('key_name')
 
         # If user_data is not specified, we don't include it in kwargs because
         # we don't want to overwrite the existing user_data.
-        include_user_data = api_version_request.is_supported(
-            req, min_version='2.57')
+        include_user_data = api_version_request.is_supported(req, '2.57')
         if include_user_data and 'user_data' in rebuild_dict:
             kwargs['user_data'] = rebuild_dict['user_data']
 
         # Skip policy check for 'rebuild:trusted_certs' if no trusted
         # certificate IDs were provided.
         if (
-            api_version_request.is_supported(req, min_version='2.63') and
+            api_version_request.is_supported(req, '2.63') and
             # Note that this is different from server create since with
             # rebuild a user can unset/reset the trusted certs by
             # specifying trusted_image_certificates=None, similar to
@@ -1200,12 +1244,12 @@ class ServersController(wsgi.Controller):
                         target=target)
 
         if (
-            api_version_request.is_supported(req, min_version='2.90') and
+            api_version_request.is_supported(req, '2.90') and
             'hostname' in rebuild_dict
         ):
             kwargs['hostname'] = rebuild_dict['hostname']
 
-        if api_version_request.is_supported(req, min_version='2.93'):
+        if api_version_request.is_supported(req, '2.93'):
             kwargs['reimage_boot_volume'] = True
 
         for request_attribute, instance_attribute in attr_map.items():
@@ -1257,15 +1301,12 @@ class ServersController(wsgi.Controller):
 
         # NOTE(liuyulong): set the new key_name for the API response.
         # from microversion 2.54 onwards.
-        show_keypair = api_version_request.is_supported(
-                           req, min_version='2.54')
-        show_server_groups = api_version_request.is_supported(
-                           req, min_version='2.71')
+        show_keypair = api_version_request.is_supported(req, '2.54')
+        show_server_groups = api_version_request.is_supported(req, '2.71')
 
         # NOTE(gmann): Starting from microversion 2.75, PUT and Rebuild
         # API response will show all attributes like GET /servers API.
-        show_all_attributes = api_version_request.is_supported(
-            req, min_version='2.75')
+        show_all_attributes = api_version_request.is_supported(req, '2.75')
         extend_address = show_all_attributes
         show_AZ = show_all_attributes
         show_config_drive = show_all_attributes
@@ -1311,8 +1352,11 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((400, 403, 404, 409))
     @wsgi.action('createImage')
-    @validation.schema(schema_servers.create_image, '2.0', '2.0')
-    @validation.schema(schema_servers.create_image, '2.1')
+    @validation.schema(schema.create_image, '2.0', '2.0')
+    @validation.schema(schema.create_image, '2.1')
+    @validation.response_body_schema(
+        schema.create_image_response, '2.0', '2.44')
+    @validation.response_body_schema(schema.create_image_response_v245, '2.45')
     def _action_create_image(self, req, id, body):
         """Snapshot a server instance."""
         context = req.environ['nova.context']
@@ -1326,9 +1370,7 @@ class ServersController(wsgi.Controller):
         metadata = entity.get('metadata', {})
 
         # Starting from microversion 2.39 we don't check quotas on createImage
-        if api_version_request.is_supported(
-                req, max_version=
-                api_version_request.MAX_IMAGE_META_PROXY_API_VERSION):
+        if not api_version_request.is_supported(req, '2.39'):
             common.check_img_metadata_properties_quota(context, metadata)
 
         bdms = objects.BlockDeviceMappingList.get_by_instance_uuid(
@@ -1353,6 +1395,8 @@ class ServersController(wsgi.Controller):
         except exception.InstanceInvalidState as state_error:
             common.raise_http_conflict_for_instance_invalid_state(state_error,
                         'createImage', id)
+        except exception.InstanceQuiesceFailed as err:
+            raise exc.HTTPConflict(explanation=err.format_message())
         except exception.Invalid as err:
             raise exc.HTTPBadRequest(explanation=err.format_message())
         except exception.OverQuota as e:
@@ -1388,20 +1432,20 @@ class ServersController(wsgi.Controller):
         # probably not trivial.
         opt_list = ('reservation_id', 'name', 'status', 'image', 'flavor',
                     'ip', 'changes-since', 'all_tenants')
-        if api_version_request.is_supported(req, min_version='2.5'):
+        if api_version_request.is_supported(req, '2.5'):
             opt_list += ('ip6',)
-        if api_version_request.is_supported(req, min_version='2.26'):
+        if api_version_request.is_supported(req, '2.26'):
             opt_list += TAG_SEARCH_FILTERS
-        if api_version_request.is_supported(req, min_version='2.66'):
+        if api_version_request.is_supported(req, '2.66'):
             opt_list += ('changes-before',)
-        if api_version_request.is_supported(req, min_version='2.73'):
+        if api_version_request.is_supported(req, '2.73'):
             opt_list += ('locked',)
-        if api_version_request.is_supported(req, min_version='2.83'):
+        if api_version_request.is_supported(req, '2.83'):
             opt_list += ('availability_zone', 'config_drive', 'key_name',
                          'created_at', 'launched_at', 'terminated_at',
                          'power_state', 'task_state', 'vm_state', 'progress',
                          'user_id',)
-        if api_version_request.is_supported(req, min_version='2.90'):
+        if api_version_request.is_supported(req, '2.90'):
             opt_list += ('hostname',)
         return opt_list
 
@@ -1420,6 +1464,8 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((404, 409))
     @wsgi.action('os-start')
+    @validation.schema(schema.start_server)
+    @validation.response_body_schema(schema.start_server_response)
     def _start_server(self, req, id, body):
         """Start an instance."""
         context = req.environ['nova.context']
@@ -1438,6 +1484,8 @@ class ServersController(wsgi.Controller):
     @wsgi.response(202)
     @wsgi.expected_errors((404, 409))
     @wsgi.action('os-stop')
+    @validation.schema(schema.stop_server)
+    @validation.response_body_schema(schema.stop_server_response)
     def _stop_server(self, req, id, body):
         """Stop an instance."""
         context = req.environ['nova.context']
@@ -1450,14 +1498,16 @@ class ServersController(wsgi.Controller):
         except (exception.InstanceNotReady, exception.InstanceIsLocked) as e:
             raise webob.exc.HTTPConflict(explanation=e.format_message())
         except exception.InstanceInvalidState as state_error:
-            common.raise_http_conflict_for_instance_invalid_state(state_error,
-                'stop', id)
+            common.raise_http_conflict_for_instance_invalid_state(
+                state_error, 'stop', id
+            )
 
-    @wsgi.Controller.api_version("2.17")
+    @wsgi.api_version("2.17")
     @wsgi.response(202)
     @wsgi.expected_errors((400, 404, 409))
     @wsgi.action('trigger_crash_dump')
-    @validation.schema(schema_servers.trigger_crash_dump)
+    @validation.schema(schema.trigger_crash_dump)
+    @validation.response_body_schema(schema.trigger_crash_dump_response)
     def _action_trigger_crash_dump(self, req, id, body):
         """Trigger crash dump in an instance"""
         context = req.environ['nova.context']
@@ -1470,8 +1520,9 @@ class ServersController(wsgi.Controller):
         except (exception.InstanceNotReady, exception.InstanceIsLocked) as e:
             raise webob.exc.HTTPConflict(explanation=e.format_message())
         except exception.InstanceInvalidState as state_error:
-            common.raise_http_conflict_for_instance_invalid_state(state_error,
-                'trigger_crash_dump', id)
+            common.raise_http_conflict_for_instance_invalid_state(
+                state_error, 'trigger_crash_dump', id
+            )
 
 
 def remove_invalid_options(context, search_options, allowed_search_options):
