@@ -15,8 +15,6 @@
 from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.functional.api import client
-from nova.tests.unit import cast_as_call
-from nova.tests.unit import policy_fixture
 
 
 class TestServerGet(test.TestCase):
@@ -24,7 +22,7 @@ class TestServerGet(test.TestCase):
 
     def setUp(self):
         super(TestServerGet, self).setUp()
-        self.useFixture(policy_fixture.RealPolicyFixture())
+        self.useFixture(nova_fixtures.RealPolicyFixture())
         self.useFixture(nova_fixtures.NeutronFixture(self))
         self.useFixture(nova_fixtures.GlanceFixture(self))
         api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
@@ -36,7 +34,7 @@ class TestServerGet(test.TestCase):
         self.start_service('scheduler')
         self.compute = self.start_service('compute')
 
-        self.useFixture(cast_as_call.CastAsCall(self))
+        self.useFixture(nova_fixtures.CastAsCallFixture(self))
 
         self.image_id = self.api.get_images()[0]['id']
         self.flavor_id = self.api.get_flavors()[0]['id']
