@@ -13,13 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from unittest import mock
-
-import fixtures as fx
-import testtools
-
-from nova.tests import fixtures
-
 """Test request logging middleware under various conditions.
 
 The request logging middleware is needed when running under something
@@ -33,6 +26,13 @@ All these tests use GET / to hit an endpoint that doesn't require the
 database setup. We have to do a bit of mocking to make that work.
 """
 
+from unittest import mock
+
+import fixtures as fx
+import testtools
+
+from nova.tests import fixtures
+
 
 class TestRequestLogMiddleware(testtools.TestCase):
 
@@ -58,7 +58,7 @@ class TestRequestLogMiddleware(testtools.TestCase):
         """
 
         emit.return_value = True
-        conf = self.useFixture(fixtures.ConfFixture()).conf
+        self.useFixture(fixtures.ConfFixture())
         self.useFixture(fixtures.RPCFixture('nova.test'))
         api = self.useFixture(fixtures.OSAPIFixture()).api
 
@@ -73,25 +73,6 @@ class TestRequestLogMiddleware(testtools.TestCase):
                 '"GET /" status: 200 len: %s' % content_length)
         self.assertIn(log1, self.stdlog.logger.output)
 
-        # Verify handling of X-Forwarded-For header, example: load balancer.
-        # First, try without setting CONF.api.use_forwarded_for, it should not
-        # use the header value.
-        headers = {'X-Forwarded-For': '1.2.3.4'}
-        resp = api.api_request('/', strip_version=True, headers=headers)
-        content_length = resp.headers['content-length']
-        log2 = ('INFO [nova.api.openstack.requestlog] 127.0.0.1 '
-                '"GET /" status: 200 len: %s' % content_length)
-        self.assertIn(log2, self.stdlog.logger.output)
-
-        # Now set CONF.api.use_forwarded_for, it should use the header value.
-        conf.set_override('use_forwarded_for', True, 'api')
-        headers = {'X-Forwarded-For': '1.2.3.4'}
-        resp = api.api_request('/', strip_version=True, headers=headers)
-        content_length = resp.headers['content-length']
-        log3 = ('INFO [nova.api.openstack.requestlog] 1.2.3.4 '
-                '"GET /" status: 200 len: %s' % content_length)
-        self.assertIn(log3, self.stdlog.logger.output)
-
     @mock.patch('nova.api.openstack.requestlog.RequestLog._should_emit')
     def test_logs_mv(self, emit):
         """Ensure logs register microversion if passed.
