@@ -54,8 +54,9 @@ class DeferredDeletePolicyTest(base.BasePolicyTest):
         # delete or restore server.
         self.project_member_authorized_contexts = [
             self.legacy_admin_context, self.system_admin_context,
-            self.project_admin_context, self.project_member_context,
-            self.project_reader_context, self.project_foo_context]
+            self.project_admin_context, self.project_manager_context,
+            self.project_member_context, self.project_reader_context,
+            self.project_foo_context]
 
     @mock.patch('nova.compute.api.API.restore')
     def test_restore_server_policy(self, mock_restore):
