@@ -15,6 +15,7 @@
 from os_brick import encryptors
 from unittest import mock
 
+import ddt
 from oslo_serialization import jsonutils
 from oslo_utils.fixture import uuidsentinel as uuids
 
@@ -35,6 +36,7 @@ from nova.volume import cinder
 ATTACHMENT_ID = uuids.attachment_id
 
 
+@ddt.ddt
 class TestDriverBlockDevice(test.NoDBTestCase):
     # os-brick>=5.1 now uses external file system locks instead of internal
     # locks so we need to set up locking
@@ -62,12 +64,20 @@ class TestDriverBlockDevice(test.NoDBTestCase):
          'guest_format': 'swap',
          'disk_bus': 'scsi',
          'volume_size': 2,
-         'boot_index': -1})
+         'boot_index': -1,
+         'encrypted': False,
+         'encryption_secret_uuid': None,
+         'encryption_format': None,
+         'encryption_options': None})
 
     swap_driver_bdm = {
         'device_name': '/dev/sdb1',
         'swap_size': 2,
-        'disk_bus': 'scsi'}
+        'disk_bus': 'scsi',
+        'encrypted': False,
+        'encryption_secret_uuid': None,
+        'encryption_format': None,
+        'encryption_options': None}
 
     ephemeral_bdm_dict = block_device.BlockDeviceDict(
         {'id': 2, 'instance_uuid': uuids.instance,
@@ -366,29 +376,49 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                               cls,
                               getattr(self, '%s_bdm' % name))
 
+        def assign_fake_changed_value(test_bdm, field, alias=None):
+            # We can't set fake values on enums, like device_type,
+            # so skip those.
+            if not isinstance(test_bdm._bdm_obj.fields[field],
+                              fields.BaseEnumField):
+                field_or_alias = alias or field
+                if field_or_alias == 'attachment_id':
+                    # Must set UUID values on UUID fields.
+                    fake_value = ATTACHMENT_ID
+                elif isinstance(test_bdm._bdm_obj.fields[field],
+                                fields.UUIDField):
+                    # Generically handle other UUID fields.
+                    fake_value = uuids.fake_value
+                elif isinstance(test_bdm._bdm_obj.fields[field],
+                                fields.BooleanField):
+                    fake_value = not test_bdm[field_or_alias]
+                else:
+                    fake_value = 'fake_changed_value'
+                test_bdm[field_or_alias] = fake_value
+
         # Test the save method
         with mock.patch.object(test_bdm._bdm_obj, 'save') as save_mock:
+            # First test saveable fields.
             for fld, alias in test_bdm._update_on_save.items():
-                # We can't set fake values on enums, like device_type,
-                # so skip those.
-                if not isinstance(test_bdm._bdm_obj.fields[fld],
-                                  fields.BaseEnumField):
-                    field = alias or fld
-                    if field == 'attachment_id':
-                        # Must set UUID values on UUID fields.
-                        fake_value = ATTACHMENT_ID
-                    elif isinstance(test_bdm._bdm_obj.fields[fld],
-                                    fields.UUIDField):
-                        # Generically handle other UUID fields.
-                        fake_value = uuids.fake_value
-                    else:
-                        fake_value = 'fake_changed_value'
-                    test_bdm[field] = fake_value
+                assign_fake_changed_value(test_bdm, fld, alias=alias)
+
             test_bdm.save()
+
             for fld, alias in test_bdm._update_on_save.items():
                 self.assertEqual(test_bdm[alias or fld],
                                  getattr(test_bdm._bdm_obj, fld))
 
+            # Then test read-only fields.
+            for field in test_bdm._readonly_fields:
+                value = test_bdm[field]
+
+                assign_fake_changed_value(test_bdm, field)
+                self.assertRaises(AttributeError, test_bdm.save)
+
+                # Reset to original value so this already tested field won't
+                # be considered during the next save().
+                test_bdm[field] = value
+
             save_mock.assert_called_once_with()
 
         def check_save():
@@ -613,6 +643,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
             # First call to get() fails because the API isn't new enough.
             # So we fallback to the old call.
             self.volume_api.get.side_effect = [
+                exception.CinderAPIVersionNotAvailable(version='3.69'),
                 exception.CinderAPIVersionNotAvailable(version='3.48'),
                 fake_volume]
 
@@ -688,14 +719,17 @@ class TestDriverBlockDevice(test.NoDBTestCase):
 
         if include_shared_targets:
             self.volume_api.get.assert_called_once_with(
-                self.context, fake_volume['id'], microversion='3.48')
+                self.context, fake_volume['id'], microversion='3.69')
         else:
             # First call to get() fails because the API isn't new enough.
             # So we fallback to the old call.
             self.volume_api.get.assert_has_calls([
+                mock.call(self.context, fake_volume['id'],
+                          microversion='3.69'),
                 mock.call(self.context, fake_volume['id'],
                           microversion='3.48'),
-                mock.call(self.context, fake_volume['id'])])
+                mock.call(self.context, fake_volume['id'],
+                          microversion=None)])
 
         try:
             self.volume_api.check_availability_zone.assert_called_once_with(
@@ -1049,7 +1083,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
             self.assertEqual('fake-volume-id-2', test_bdm.volume_id)
             mock_attach.assert_called_once_with(
                 self.context, instance, self.volume_api, self.virt_driver)
-            # Make sure theses are not called
+            # Make sure these are not called
             self.volume_api.get_snapshot.assert_not_called()
             self.volume_api.create.assert_not_called()
 
@@ -1182,7 +1216,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
             self.assertEqual('fake-volume-id-2', test_bdm.volume_id)
             mock_attach.assert_called_once_with(
                 self.context, instance, self.volume_api, self.virt_driver)
-            # Make sure theses are not called
+            # Make sure these are not called
             self.volume_api.get_snapshot.assert_not_called()
             self.volume_api.create.assert_not_called()
 
@@ -1557,6 +1591,24 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self._test_boot_from_volume_source_snapshot_volume_type(
             bdm, 'fake-lvm-1')
 
+    @ddt.data(['3.69'], ['3.69', '3.48'], ['3.69', '3.48', None])
+    def test__get_volume(self, microversions):
+        volume_api = mock.Mock()
+        exp = mock.Mock()
+        exc = exception.CinderAPIVersionNotAvailable
+        side_effect = [exc(version=mv) for mv in microversions[:-1]] + [exp]
+        volume_api.get.side_effect = side_effect
+
+        res = self.driver_classes['volume']._get_volume(
+            self.context, volume_api, mock.sentinel.volume_id)
+
+        self.assertEqual(exp, res)
+
+        self.assertEqual(len(microversions), volume_api.get.call_count)
+        volume_api.get.assert_has_calls(
+            [mock.call(self.context, mock.sentinel.volume_id, microversion=mv)
+             for mv in microversions])
+
 
 class TestDriverBlockDeviceNewFlow(TestDriverBlockDevice):
     """Virt block_device tests for the Cinder 3.44 volume attach flow
