@@ -11,6 +11,7 @@
 #    under the License.
 
 import re
+import warnings
 
 import nova.conf
 from nova.virt.libvirt import utils as libvirt_utils
@@ -25,6 +26,11 @@ USERNAME_REGEX = re.compile(r"(user(?:name)?)=(?:[^ ,]+\\)?([^ ,]+)")
 class LibvirtSMBFSVolumeDriver(fs.LibvirtBaseFileSystemVolumeDriver):
     """Class implements libvirt part of volume driver for SMBFS."""
 
+    def __init__(self, host):
+        super(LibvirtSMBFSVolumeDriver, self).__init__(host)
+        warnings.warn('SMBFS volume support is deprecated',
+                      category=DeprecationWarning, stacklevel=2)
+
     def _get_mount_point_base(self):
         return CONF.libvirt.smbfs_mount_point_base
 
@@ -34,12 +40,12 @@ class LibvirtSMBFSVolumeDriver(fs.LibvirtBaseFileSystemVolumeDriver):
                      self).get_config(connection_info, disk_info)
 
         conf.source_type = 'file'
-        conf.driver_cache = 'writethrough'
+        conf.driver_cache = 'writeback'
         conf.source_path = connection_info['data']['device_path']
         conf.driver_format = connection_info['data'].get('format', 'raw')
         return conf
 
-    def connect_volume(self, connection_info, disk_info):
+    def connect_volume(self, connection_info, instance):
         """Connect the volume."""
         smbfs_share = connection_info['data']['export']
         mount_path = self._get_mount_path(connection_info)
@@ -52,7 +58,7 @@ class LibvirtSMBFSVolumeDriver(fs.LibvirtBaseFileSystemVolumeDriver):
         device_path = self._get_device_path(connection_info)
         connection_info['data']['device_path'] = device_path
 
-    def disconnect_volume(self, connection_info, disk_dev):
+    def disconnect_volume(self, connection_info, instance, force=False):
         """Disconnect the volume."""
         smbfs_share = connection_info['data']['export']
         mount_path = self._get_mount_path(connection_info)
