@@ -20,13 +20,12 @@ The VMware API VM utility module to build SOAP object specs.
 
 import collections
 import copy
-import functools
 
 from oslo_log import log as logging
+from oslo_service import loopingcall
 from oslo_utils import excutils
 from oslo_utils import units
 from oslo_vmware import exceptions as vexc
-from oslo_vmware.objects import datastore as ds_obj
 from oslo_vmware import pbm
 from oslo_vmware import vim_util as vutil
 
@@ -36,6 +35,7 @@ from nova import exception
 from nova.i18n import _
 from nova.network import model as network_model
 from nova.virt.vmwareapi import constants
+from nova.virt.vmwareapi import session
 from nova.virt.vmwareapi import vim_util
 
 LOG = logging.getLogger(__name__)
@@ -118,32 +118,16 @@ def vm_refs_cache_reset():
     _VM_REFS_CACHE = {}
 
 
-def vm_ref_cache_delete(id):
-    _VM_REFS_CACHE.pop(id, None)
+def vm_ref_cache_delete(id_):
+    _VM_REFS_CACHE.pop(id_, None)
 
 
-def vm_ref_cache_update(id, vm_ref):
-    _VM_REFS_CACHE[id] = vm_ref
+def vm_ref_cache_update(id_, vm_ref):
+    _VM_REFS_CACHE[id_] = vm_ref
 
 
-def vm_ref_cache_get(id):
-    return _VM_REFS_CACHE.get(id)
-
-
-def _vm_ref_cache(id, func, session, data):
-    vm_ref = vm_ref_cache_get(id)
-    if not vm_ref:
-        vm_ref = func(session, data)
-        vm_ref_cache_update(id, vm_ref)
-    return vm_ref
-
-
-def vm_ref_cache_from_instance(func):
-    @functools.wraps(func)
-    def wrapper(session, instance):
-        id = instance.uuid
-        return _vm_ref_cache(id, func, session, instance)
-    return wrapper
+def vm_ref_cache_get(id_):
+    return _VM_REFS_CACHE.get(id_)
 
 
 # the config key which stores the VNC port
@@ -276,7 +260,7 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
     # enable to provide info needed by udev to generate /dev/disk/by-id
     opt = client_factory.create('ns0:OptionValue')
     opt.key = "disk.EnableUUID"
-    opt.value = True
+    opt.value = 'true'
     extra_config.append(opt)
 
     port_index = 0
@@ -690,46 +674,57 @@ def get_hardware_devices(session, vm_ref):
     return vim_util.get_array_items(hardware_devices)
 
 
-def get_vmdk_info(session, vm_ref, uuid=None):
-    """Returns information for the primary VMDK attached to the given VM."""
+def _is_before_in_boot_order(disk1, disk2):
+    return (disk1.controllerKey < disk2.controllerKey or
+            disk1.controllerKey == disk2.controllerKey and
+                disk1.unitNumber < disk2.unitNumber)
+
+
+def get_vmdk_info(session, vm_ref):
+    """Returns information for the first VMDK attached to the given VM.
+
+    The order follows the boot order of the hypervisor and is determined as
+    follows:
+    - if disks are attached at multiple controllers, disks at a controller with
+      a lower device number take precedence.
+    - if more than one disk is attached at the same controller disks with a
+      lower unit number will take precedence
+    """
     hardware_devices = get_hardware_devices(session, vm_ref)
     vmdk_file_path = None
     vmdk_controller_key = None
     disk_type = None
     capacity_in_bytes = 0
 
-    # Determine if we need to get the details of the root disk
-    root_disk = None
-    root_device = None
-    if uuid:
-        root_disk = '%s.vmdk' % uuid
-    vmdk_device = None
-
+    # Find the lowest_disk_device and fill the dict adapter_type_dict.
+    lowest_disk_device = None
     adapter_type_dict = {}
     for device in hardware_devices:
+        # For the disk-devices: Check if the current disk-device is before the
+        #    lowest_disk_device in the boot order
         if device.__class__.__name__ == "VirtualDisk":
-            if device.backing.__class__.__name__ == \
-                    "VirtualDiskFlatVer2BackingInfo":
-                path = ds_obj.DatastorePath.parse(device.backing.fileName)
-                if root_disk and path.basename == root_disk:
-                    root_device = device
-                vmdk_device = device
+            if (device.backing.__class__.__name__ ==
+                    "VirtualDiskFlatVer2BackingInfo"):
+                if (lowest_disk_device is None or
+                        _is_before_in_boot_order(device, lowest_disk_device)):
+                    lowest_disk_device = device
+        # For the controller devices: Memorize for controller-type as
+        # defined by the class name for the device.key. So, we can look it up
+        # easily, when we know which is actually the first.
         elif device.__class__.__name__ in CONTROLLER_TO_ADAPTER_TYPE:
             adapter_type_dict[device.key] = CONTROLLER_TO_ADAPTER_TYPE[
                 device.__class__.__name__]
 
-    if root_disk:
-        vmdk_device = root_device
-
-    if vmdk_device:
-        vmdk_file_path = vmdk_device.backing.fileName
-        capacity_in_bytes = _get_device_capacity(vmdk_device)
-        vmdk_controller_key = vmdk_device.controllerKey
-        disk_type = _get_device_disk_type(vmdk_device)
+    if lowest_disk_device:
+        vmdk_file_path = lowest_disk_device.backing.fileName
+        capacity_in_bytes = _get_device_capacity(lowest_disk_device)
+        vmdk_controller_key = lowest_disk_device.controllerKey
+        disk_type = _get_device_disk_type(lowest_disk_device)
 
+    # Get the adapter type for the disk-device (if found).
     adapter_type = adapter_type_dict.get(vmdk_controller_key)
     return VmdkInfo(vmdk_file_path, adapter_type, disk_type,
-                    capacity_in_bytes, vmdk_device)
+                    capacity_in_bytes, lowest_disk_device)
 
 
 scsi_controller_classes = {
@@ -744,9 +739,6 @@ def get_scsi_adapter_type(hardware_devices):
     """Selects a proper iscsi adapter type from the existing
        hardware devices
     """
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
-
     for device in hardware_devices:
         if device.__class__.__name__ in scsi_controller_classes:
             # find the controllers which still have available slots
@@ -840,9 +832,6 @@ def allocate_controller_key_and_unit_number(client_factory, devices,
 
 def get_rdm_disk(hardware_devices, uuid):
     """Gets the RDM disk key."""
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
-
     for device in hardware_devices:
         if (device.__class__.__name__ == "VirtualDisk" and
             device.backing.__class__.__name__ ==
@@ -1067,41 +1056,33 @@ def _get_allocated_vnc_ports(session):
     vnc_ports = set()
     result = session._call_method(vim_util, "get_objects",
                                   "VirtualMachine", [VNC_CONFIG_KEY])
-    while result:
-        for obj in result.objects:
-            if not hasattr(obj, 'propSet'):
+    with vutil.WithRetrieval(session.vim, result) as objects:
+        for obj in objects:
+            if not hasattr(obj, 'propSet') or not obj.propSet:
                 continue
             dynamic_prop = obj.propSet[0]
             option_value = dynamic_prop.val
             vnc_port = option_value.value
             vnc_ports.add(int(vnc_port))
-        result = session._call_method(vutil, 'continue_retrieval',
-                                      result)
     return vnc_ports
 
 
-def _get_object_for_value(results, value):
-    for object in results.objects:
+def _get_object_for_value(objects, value):
+    for object in objects:
         if object.propSet[0].val == value:
             return object.obj
 
 
-def _get_object_for_optionvalue(results, value):
-    for object in results.objects:
+def _get_object_for_optionvalue(objects, value):
+    for object in objects:
         if hasattr(object, "propSet") and object.propSet:
             if object.propSet[0].val.value == value:
                 return object.obj
 
 
 def _get_object_from_results(session, results, value, func):
-    while results:
-        object = func(results, value)
-        if object:
-            session._call_method(vutil, 'cancel_retrieval',
-                                 results)
-            return object
-        results = session._call_method(vutil, 'continue_retrieval',
-                                       results)
+    with vutil.WithRetrieval(session.vim, results) as objects:
+        return func(objects, value)
 
 
 def _get_vm_ref_from_name(session, vm_name):
@@ -1144,15 +1125,25 @@ def _get_vm_ref_from_extraconfig(session, instance_uuid):
                                      _get_object_for_optionvalue)
 
 
-@vm_ref_cache_from_instance
+class VmMoRefProxy(session.StableMoRefProxy):
+    def __init__(self, ref, uuid):
+        super(VmMoRefProxy, self).__init__(ref)
+        self._uuid = uuid
+
+    def fetch_moref(self, session):
+        self.moref = search_vm_ref_by_identifier(session, self._uuid)
+        if not self.moref:
+            raise exception.InstanceNotFound(instance_id=self._uuid)
+        vm_ref_cache_update(self._uuid, self.moref)
+
+
 def get_vm_ref(session, instance):
-    """Get reference to the VM through uuid or vm name."""
-    uuid = instance.uuid
-    vm_ref = (search_vm_ref_by_identifier(session, uuid) or
-              _get_vm_ref_from_name(session, instance.name))
-    if vm_ref is None:
-        raise exception.InstanceNotFound(instance_id=uuid)
-    return vm_ref
+    """Get reference to the VM through uuid."""
+    moref = vm_ref_cache_get(instance.uuid)
+    stable_ref = VmMoRefProxy(moref, instance.uuid)
+    if not moref:
+        stable_ref.fetch_moref(session)
+    return stable_ref
 
 
 def search_vm_ref_by_identifier(session, identifier):
@@ -1164,8 +1155,7 @@ def search_vm_ref_by_identifier(session, identifier):
     use get_vm_ref instead.
     """
     vm_ref = (_get_vm_ref_from_vm_uuid(session, identifier) or
-              _get_vm_ref_from_extraconfig(session, identifier) or
-              _get_vm_ref_from_name(session, identifier))
+              _get_vm_ref_from_extraconfig(session, identifier))
     return vm_ref
 
 
@@ -1213,13 +1203,15 @@ def get_stats_from_cluster(session, cluster):
                          "HostSystem", host_mors,
                          ["summary.hardware", "summary.runtime",
                           "summary.quickStats"])
-            for obj in result.objects:
-                host_props = propset_dict(obj.propSet)
-                hardware_summary = host_props['summary.hardware']
-                runtime_summary = host_props['summary.runtime']
-                stats_summary = host_props['summary.quickStats']
-                if (runtime_summary.inMaintenanceMode is False and
-                    runtime_summary.connectionState == "connected"):
+            with vutil.WithRetrieval(session.vim, result) as objects:
+                for obj in objects:
+                    host_props = propset_dict(obj.propSet)
+                    runtime_summary = host_props['summary.runtime']
+                    if (runtime_summary.inMaintenanceMode is not False or
+                            runtime_summary.connectionState != "connected"):
+                        continue
+                    hardware_summary = host_props['summary.hardware']
+                    stats_summary = host_props['summary.quickStats']
                     # Total vcpus is the sum of all pCPUs of individual hosts
                     # The overcommitment ratio is factored in by the scheduler
                     threads = hardware_summary.numCpuThreads
@@ -1277,9 +1269,6 @@ def propset_dict(propset):
 
 
 def get_vmdk_backed_disk_device(hardware_devices, uuid):
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
-
     for device in hardware_devices:
         if (device.__class__.__name__ == "VirtualDisk" and
                 device.backing.__class__.__name__ ==
@@ -1289,9 +1278,6 @@ def get_vmdk_backed_disk_device(hardware_devices, uuid):
 
 
 def get_vmdk_volume_disk(hardware_devices, path=None):
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
-
     for device in hardware_devices:
         if (device.__class__.__name__ == "VirtualDisk"):
             if not path or path == device.backing.fileName:
@@ -1313,13 +1299,8 @@ def get_all_cluster_mors(session):
     try:
         results = session._call_method(vim_util, "get_objects",
                                         "ClusterComputeResource", ["name"])
-        session._call_method(vutil, 'cancel_retrieval',
-                             results)
-        if results.objects is None:
-            return []
-        else:
-            return results.objects
-
+        with vutil.WithRetrieval(session.vim, results) as objects:
+            return list(objects)
     except Exception as excep:
         LOG.warning("Failed to get cluster references %s", excep)
 
@@ -1328,7 +1309,7 @@ def get_cluster_ref_by_name(session, cluster_name):
     """Get reference to the vCenter cluster with the specified name."""
     all_clusters = get_all_cluster_mors(session)
     for cluster in all_clusters:
-        if (hasattr(cluster, 'propSet') and
+        if (hasattr(cluster, 'propSet') and cluster.propSet and
                     cluster.propSet[0].val == cluster_name):
             return cluster.obj
 
@@ -1348,6 +1329,9 @@ def get_vmdk_adapter_type(adapter_type):
     return vmdk_adapter_type
 
 
+@loopingcall.RetryDecorator(
+    max_retry_count=20, inc_sleep_time=2, max_sleep_time=20,
+    exceptions=(vexc.VimFaultException,))
 def create_vm(session, instance, vm_folder, config_spec, res_pool_ref):
     """Create VM on ESX host."""
     LOG.debug("Creating VM on the ESX host", instance=instance)
@@ -1539,12 +1523,12 @@ def find_rescue_device(hardware_devices, instance):
     """Returns the rescue device.
 
     The method will raise an exception if the rescue device does not
-    exist. The resuce device has suffix '-rescue.vmdk'.
+    exist. The rescue device has suffix '-rescue.vmdk'.
     :param hardware_devices: the hardware devices for the instance
     :param instance: nova.objects.instance.Instance object
     :return: the rescue disk device object
     """
-    for device in hardware_devices.VirtualDevice:
+    for device in hardware_devices:
         if (device.__class__.__name__ == "VirtualDisk" and
                 device.backing.__class__.__name__ ==
                 'VirtualDiskFlatVer2BackingInfo' and
@@ -1555,8 +1539,8 @@ def find_rescue_device(hardware_devices, instance):
     raise exception.NotFound(msg)
 
 
-def get_ephemeral_name(id):
-    return 'ephemeral_%d.vmdk' % id
+def get_ephemeral_name(id_):
+    return 'ephemeral_%d.vmdk' % id_
 
 
 def _detach_and_delete_devices_config_spec(client_factory, devices):
@@ -1583,13 +1567,7 @@ def detach_devices_from_vm(session, vm_ref, devices):
 
 def get_ephemerals(session, vm_ref):
     devices = []
-    hardware_devices = session._call_method(vutil,
-                                            "get_object_property",
-                                            vm_ref,
-                                            "config.hardware.device")
-
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
+    hardware_devices = get_hardware_devices(session, vm_ref)
 
     for device in hardware_devices:
         if device.__class__.__name__ == "VirtualDisk":
@@ -1601,11 +1579,7 @@ def get_ephemerals(session, vm_ref):
 
 
 def get_swap(session, vm_ref):
-    hardware_devices = session._call_method(vutil, "get_object_property",
-                                            vm_ref, "config.hardware.device")
-
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
+    hardware_devices = get_hardware_devices(session, vm_ref)
 
     for device in hardware_devices:
         if (device.__class__.__name__ == "VirtualDisk" and
@@ -1623,15 +1597,18 @@ def create_folder(session, parent_folder_ref, name):
     """
 
     LOG.debug("Creating folder: %(name)s. Parent ref: %(parent)s.",
-              {'name': name, 'parent': parent_folder_ref.value})
+              {'name': name,
+               'parent': vutil.get_moref_value(parent_folder_ref)})
     try:
         folder = session._call_method(session.vim, "CreateFolder",
                                       parent_folder_ref, name=name)
         LOG.info("Created folder: %(name)s in parent %(parent)s.",
-                 {'name': name, 'parent': parent_folder_ref.value})
+                 {'name': name,
+                  'parent': vutil.get_moref_value(parent_folder_ref)})
     except vexc.DuplicateName as e:
         LOG.debug("Folder already exists: %(name)s. Parent ref: %(parent)s.",
-                  {'name': name, 'parent': parent_folder_ref.value})
+                  {'name': name,
+                   'parent': vutil.get_moref_value(parent_folder_ref)})
         val = e.details['object']
         folder = vutil.get_moref(val, 'Folder')
     return folder
@@ -1645,11 +1622,11 @@ def folder_ref_cache_get(path):
     return _FOLDER_PATH_REF_MAPPING.get(path)
 
 
-def _get_vm_name(display_name, id):
+def _get_vm_name(display_name, id_):
     if display_name:
-        return '%s (%s)' % (display_name[:41], id[:36])
-    else:
-        return id[:36]
+        return '%s (%s)' % (display_name[:41], id_[:36])
+
+    return id_[:36]
 
 
 def rename_vm(session, vm_ref, instance):
@@ -1657,3 +1634,36 @@ def rename_vm(session, vm_ref, instance):
     rename_task = session._call_method(session.vim, "Rename_Task", vm_ref,
                                        newName=vm_name)
     session._wait_for_task(rename_task)
+
+
+def _create_fcd_id_obj(client_factory, fcd_id):
+    id_obj = client_factory.create('ns0:ID')
+    id_obj.id = fcd_id
+    return id_obj
+
+
+def attach_fcd(
+        session, vm_ref, fcd_id, ds_ref_val, controller_key, unit_number
+    ):
+    client_factory = session.vim.client.factory
+    disk_id = _create_fcd_id_obj(client_factory, fcd_id)
+    ds_ref = vutil.get_moref(ds_ref_val, 'Datastore')
+    LOG.debug("Attaching fcd (id: %(fcd_id)s, datastore: %(ds_ref_val)s) to "
+              "vm: %(vm_ref)s.",
+              {'fcd_id': fcd_id,
+               'ds_ref_val': ds_ref_val,
+               'vm_ref': vm_ref})
+    task = session._call_method(
+        session.vim, "AttachDisk_Task", vm_ref, diskId=disk_id,
+        datastore=ds_ref, controllerKey=controller_key, unitNumber=unit_number)
+    session._wait_for_task(task)
+
+
+def detach_fcd(session, vm_ref, fcd_id):
+    client_factory = session.vim.client.factory
+    disk_id = _create_fcd_id_obj(client_factory, fcd_id)
+    LOG.debug("Detaching fcd (id: %(fcd_id)s) from vm: %(vm_ref)s.",
+              {'fcd_id': fcd_id, 'vm_ref': vm_ref})
+    task = session._call_method(
+        session.vim, "DetachDisk_Task", vm_ref, diskId=disk_id)
+    session._wait_for_task(task)
